@@ -1,5 +1,5 @@
 /* nm.c -- Describe symbol table of a rel file.
-   Copyright (C) 1991-2022 Free Software Foundation, Inc.
+   Copyright (C) 1991-2023 Free Software Foundation, Inc.
 
    This file is part of GNU Binutils.
 
@@ -20,7 +20,6 @@
 
 #include "sysdep.h"
 #include "bfd.h"
-#include "progress.h"
 #include "getopt.h"
 #include "aout/stab_gnu.h"
 #include "aout/ranlib.h"
@@ -54,15 +53,16 @@ struct size_sym
   bfd_vma size;
 };
 
-/* When fetching relocs, we use this structure to pass information to
-   get_relocs.  */
+/* line number related info cached in bfd usrdata.  */
 
-struct get_relocs_info
+struct lineno_cache
 {
   asection **secs;
   arelent ***relocs;
   long *relcount;
   asymbol **syms;
+  long symcount;
+  unsigned int seccount;
 };
 
 struct extended_symbol_info
@@ -219,10 +219,6 @@ static const char *plugin_target = "plugin";
 static const char *plugin_target = NULL;
 #endif
 
-/* Used to cache the line numbers for a BFD.  */
-static bfd *lineno_cache_bfd;
-static bfd *lineno_cache_rel_bfd;
-
 typedef enum unicode_display_type
 {
   unicode_default = 0,
@@ -741,19 +737,19 @@ print_symdef_entry (bfd *abfd)
        idx != BFD_NO_MORE_SYMBOLS;
        idx = bfd_get_next_mapent (abfd, idx, &thesym))
     {
-      bfd *elt;
       if (!everprinted)
 	{
 	  printf (_("\nArchive index:\n"));
 	  everprinted = true;
 	}
-      elt = bfd_get_elt_at_index (abfd, idx);
-      if (elt == NULL)
-	bfd_fatal ("bfd_get_elt_at_index");
-      if (thesym->name != (char *) NULL)
+      if (thesym->name != NULL)
 	{
 	  print_symname ("%s", NULL, thesym->name, abfd);
-	  printf (" in %s\n", bfd_get_filename (elt));
+	  bfd *elt = bfd_get_elt_at_index (abfd, idx);
+	  if (elt)
+	    printf (" in %s\n", bfd_get_filename (elt));
+	  else
+	    printf ("\n");
 	}
     }
 }
@@ -786,11 +782,9 @@ filter_symbols (bfd *abfd, bool is_dynamic, void *minisyms,
       int keep = 0;
       asymbol *sym;
 
-      PROGRESS (1);
-
-      sym = bfd_minisymbol_to_symbol (abfd, is_dynamic, (const void *) from, store);
+      sym = bfd_minisymbol_to_symbol (abfd, is_dynamic, from, store);
       if (sym == NULL)
-	bfd_fatal (bfd_get_filename (abfd));
+	continue;
 
       if (sym->name != NULL
 	  && sym->name[0] == '_'
@@ -1143,28 +1137,21 @@ sort_symbols_by_size (bfd *abfd, bool is_dynamic, void *minisyms,
 static void
 get_relocs (bfd *abfd, asection *sec, void *dataarg)
 {
-  struct get_relocs_info *data = (struct get_relocs_info *) dataarg;
+  struct lineno_cache *data = (struct lineno_cache *) dataarg;
 
   *data->secs = sec;
+  *data->relocs = NULL;
+  *data->relcount = 0;
 
-  if ((sec->flags & SEC_RELOC) == 0)
+  if ((sec->flags & SEC_RELOC) != 0)
     {
-      *data->relocs = NULL;
-      *data->relcount = 0;
-    }
-  else
-    {
-      long relsize;
-
-      relsize = bfd_get_reloc_upper_bound (abfd, sec);
-      if (relsize < 0)
-	bfd_fatal (bfd_get_filename (abfd));
-
-      *data->relocs = (arelent **) xmalloc (relsize);
-      *data->relcount = bfd_canonicalize_reloc (abfd, sec, *data->relocs,
-						data->syms);
-      if (*data->relcount < 0)
-	bfd_fatal (bfd_get_filename (abfd));
+      long relsize = bfd_get_reloc_upper_bound (abfd, sec);
+      if (relsize > 0)
+	{
+	  *data->relocs = (arelent **) xmalloc (relsize);
+	  *data->relcount = bfd_canonicalize_reloc (abfd, sec, *data->relocs,
+						    data->syms);
+	}
     }
 
   ++data->secs;
@@ -1172,6 +1159,25 @@ get_relocs (bfd *abfd, asection *sec, void *dataarg)
   ++data->relcount;
 }
 
+static void
+free_lineno_cache (bfd *abfd)
+{
+  struct lineno_cache *lc = bfd_usrdata (abfd);
+
+  if (lc)
+    {
+      if (lc->relocs)
+	for (unsigned int i = 0; i < lc->seccount; i++)
+	  free (lc->relocs[i]);
+      free (lc->relcount);
+      free (lc->relocs);
+      free (lc->secs);
+      free (lc->syms);
+      free (lc);
+      bfd_set_usrdata (abfd, NULL);
+    }
+}
+
 /* Print a single symbol.  */
 
 static void
@@ -1183,8 +1189,6 @@ print_symbol (bfd *        abfd,
   symbol_info syminfo;
   struct extended_symbol_info info;
 
-  PROGRESS (1);
-
   format->print_symbol_filename (archive_bfd, abfd);
 
   bfd_get_symbol_info (abfd, sym, &syminfo);
@@ -1220,98 +1224,74 @@ print_symbol (bfd *        abfd,
 
   if (line_numbers)
     {
-      static asymbol **syms;
-      static long symcount;
+      struct lineno_cache *lc = bfd_usrdata (abfd);
       const char *filename, *functionname;
       unsigned int lineno;
 
       /* We need to get the canonical symbols in order to call
          bfd_find_nearest_line.  This is inefficient, but, then, you
          don't have to use --line-numbers.  */
-      if (abfd != lineno_cache_bfd && syms != NULL)
+      if (lc == NULL)
 	{
-	  free (syms);
-	  syms = NULL;
+	  lc = xcalloc (1, sizeof (*lc));
+	  bfd_set_usrdata (abfd, lc);
 	}
-      if (syms == NULL)
+      if (lc->syms == NULL && lc->symcount == 0)
 	{
-	  long symsize;
-
-	  symsize = bfd_get_symtab_upper_bound (abfd);
-	  if (symsize < 0)
-	    bfd_fatal (bfd_get_filename (abfd));
-	  syms = (asymbol **) xmalloc (symsize);
-	  symcount = bfd_canonicalize_symtab (abfd, syms);
-	  if (symcount < 0)
-	    bfd_fatal (bfd_get_filename (abfd));
-	  lineno_cache_bfd = abfd;
+	  long symsize = bfd_get_symtab_upper_bound (abfd);
+	  if (symsize <= 0)
+	    lc->symcount = -1;
+	  else
+	    {
+	      lc->syms = xmalloc (symsize);
+	      lc->symcount = bfd_canonicalize_symtab (abfd, lc->syms);
+	    }
 	}
 
-      if (bfd_is_und_section (bfd_asymbol_section (sym)))
+      if (lc->symcount <= 0)
+	;
+      else if (bfd_is_und_section (bfd_asymbol_section (sym)))
 	{
-	  static asection **secs;
-	  static arelent ***relocs;
-	  static long *relcount;
-	  static unsigned int seccount;
 	  unsigned int i;
 	  const char *symname;
 
 	  /* For an undefined symbol, we try to find a reloc for the
              symbol, and print the line number of the reloc.  */
-	  if (abfd != lineno_cache_rel_bfd && relocs != NULL)
+	  if (lc->relocs == NULL)
 	    {
-	      for (i = 0; i < seccount; i++)
-		if (relocs[i] != NULL)
-		  free (relocs[i]);
-	      free (secs);
-	      free (relocs);
-	      free (relcount);
-	      secs = NULL;
-	      relocs = NULL;
-	      relcount = NULL;
-	    }
+	      unsigned int seccount = bfd_count_sections (abfd);
+	      lc->seccount = seccount;
+	      lc->secs = xmalloc (seccount * sizeof (*lc->secs));
+	      lc->relocs = xmalloc (seccount * sizeof (*lc->relocs));
+	      lc->relcount = xmalloc (seccount * sizeof (*lc->relcount));
 
-	  if (relocs == NULL)
-	    {
-	      struct get_relocs_info rinfo;
-
-	      seccount = bfd_count_sections (abfd);
-
-	      secs = (asection **) xmalloc (seccount * sizeof *secs);
-	      relocs = (arelent ***) xmalloc (seccount * sizeof *relocs);
-	      relcount = (long *) xmalloc (seccount * sizeof *relcount);
-
-	      rinfo.secs = secs;
-	      rinfo.relocs = relocs;
-	      rinfo.relcount = relcount;
-	      rinfo.syms = syms;
-	      bfd_map_over_sections (abfd, get_relocs, (void *) &rinfo);
-	      lineno_cache_rel_bfd = abfd;
+	      struct lineno_cache rinfo = *lc;
+	      bfd_map_over_sections (abfd, get_relocs, &rinfo);
 	    }
 
 	  symname = bfd_asymbol_name (sym);
-	  for (i = 0; i < seccount; i++)
+	  for (i = 0; i < lc->seccount; i++)
 	    {
 	      long j;
 
-	      for (j = 0; j < relcount[i]; j++)
+	      for (j = 0; j < lc->relcount[i]; j++)
 		{
 		  arelent *r;
 
-		  r = relocs[i][j];
+		  r = lc->relocs[i][j];
 		  if (r->sym_ptr_ptr != NULL
 		      && (*r->sym_ptr_ptr)->section == sym->section
 		      && (*r->sym_ptr_ptr)->value == sym->value
 		      && strcmp (symname,
 				 bfd_asymbol_name (*r->sym_ptr_ptr)) == 0
-		      && bfd_find_nearest_line (abfd, secs[i], syms,
+		      && bfd_find_nearest_line (abfd, lc->secs[i], lc->syms,
 						r->address, &filename,
 						&functionname, &lineno)
 		      && filename != NULL)
 		    {
 		      /* We only print the first one we find.  */
 		      printf ("\t%s:%u", filename, lineno);
-		      i = seccount;
+		      i = lc->seccount;
 		      break;
 		    }
 		}
@@ -1319,9 +1299,9 @@ print_symbol (bfd *        abfd,
 	}
       else if (bfd_asymbol_section (sym)->owner == abfd)
 	{
-	  if ((bfd_find_line (abfd, syms, sym, &filename, &lineno)
+	  if ((bfd_find_line (abfd, lc->syms, sym, &filename, &lineno)
 	       || bfd_find_nearest_line (abfd, bfd_asymbol_section (sym),
-					 syms, sym->value, &filename,
+					 lc->syms, sym->value, &filename,
 					 &functionname, &lineno))
 	      && filename != NULL
 	      && lineno != 0)
@@ -1426,19 +1406,7 @@ display_rel_file (bfd *abfd, bfd *archive_bfd)
     }
 
   symcount = bfd_read_minisymbols (abfd, dynamic, &minisyms, &size);
-  if (symcount < 0)
-    {
-      if (dynamic && bfd_get_error () == bfd_error_no_symbols)
-	{
-	  if (!quiet)
-	    non_fatal (_("%s: no symbols"), bfd_get_filename (abfd));
-	  return;
-	}
-
-      bfd_fatal (bfd_get_filename (abfd));
-    }
-
-  if (symcount == 0)
+  if (symcount <= 0)
     {
       if (!quiet)
 	non_fatal (_("%s: no symbols"), bfd_get_filename (abfd));
@@ -1470,7 +1438,7 @@ display_rel_file (bfd *abfd, bfd *archive_bfd)
 	      dyn_syms = (asymbol **) xmalloc (storage);
 	      dyn_count = bfd_canonicalize_dynamic_symtab (abfd, dyn_syms);
 	      if (dyn_count < 0)
-		bfd_fatal (bfd_get_filename (abfd));
+		dyn_count = 0;
 	    }
 	}
 
@@ -1604,14 +1572,12 @@ display_archive (bfd *file)
 
   for (;;)
     {
-      PROGRESS (1);
-
       arfile = bfd_openr_next_archived_file (file, arfile);
 
       if (arfile == NULL)
 	{
 	  if (bfd_get_error () != bfd_error_no_more_archived_files)
-	    bfd_fatal (bfd_get_filename (file));
+	    bfd_nonfatal (bfd_get_filename (file));
 	  break;
 	}
 
@@ -1631,9 +1597,8 @@ display_archive (bfd *file)
 
       if (last_arfile != NULL)
 	{
+	  free_lineno_cache (last_arfile);
 	  bfd_close (last_arfile);
-	  lineno_cache_bfd = NULL;
-	  lineno_cache_rel_bfd = NULL;
 	  if (arfile == last_arfile)
 	    return;
 	}
@@ -1642,9 +1607,8 @@ display_archive (bfd *file)
 
   if (last_arfile != NULL)
     {
+      free_lineno_cache (last_arfile);
       bfd_close (last_arfile);
-      lineno_cache_bfd = NULL;
-      lineno_cache_rel_bfd = NULL;
     }
 }
 
@@ -1687,11 +1651,9 @@ display_file (char *filename)
       retval = false;
     }
 
+  free_lineno_cache (file);
   if (!bfd_close (file))
-    bfd_fatal (filename);
-
-  lineno_cache_bfd = NULL;
-  lineno_cache_rel_bfd = NULL;
+    retval = false;
 
   return retval;
 }
@@ -2005,8 +1967,6 @@ main (int argc, char **argv)
   bfd_plugin_set_program_name (program_name);
 #endif
 
-  START_PROGRESS (program_name, 0);
-
   expandargv (&argc, &argv);
 
   if (bfd_init () != BFD_INIT_MAGIC)
@@ -2193,13 +2153,10 @@ main (int argc, char **argv)
   /* We were given several filenames to do.  */
   while (optind < argc)
     {
-      PROGRESS (1);
       if (!display_file (argv[optind++]))
 	retval++;
     }
 
-  END_PROGRESS (program_name);
-
   exit (retval);
   return retval;
 }
